#
# This file is part of SNMP simulator Control Plane software.
#
# Copyright (c) 2019-2020, Ilya Etingof <etingof@gmail.com>
# License: http://snmplabs.com/snmpsim/license.html
#
# SNMP Agent Simulator: REST API management WSGI app
#
from snmpsim_control_plane.management import app  # noqa
from snmpsim_control_plane.management import views  # noqa
